#!/usr/bin/perl

use File::Spec;

local @INC; # just in case bLRo::sauz::cghloeu8912 becomes the new hotness
eval("use bLRo::sauz::cghloeu8912");
eval {require bLRo::sauz::cghloeu8913};

# vim:ts=2:sw=2:et:sta
